﻿using System; 
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using HeuristicLab.Services.Optimization.Web.Models;
using HeuristicLab.Services.Optimization.Web.Helpers;
using HeuristicLab.Services.Optimization.ControllerService.Model;
using System.Web.Security;

namespace HeuristicLab.Services.Optimization.Web.Controllers
{
    [Authorize(Roles = "Web User")]
    public class ExperimentController : Controller
    {
        private ControllerServiceHelper ControllerService {
          get {
            var helper = new ControllerServiceHelper(Session["pw"] as string);
            if (!helper.Valid) {
              Response.Redirect("/Account/Logon?ReturnUrl=%2fExperiment");
            }
            return helper;
          }
        }

        //
        // GET: /Experiment/

        public ActionResult Index()
        {
          var model = new ExperimentViewModel();
          var scenarios = ControllerService.withControllerService<IEnumerable<string>>((service) => {
            return service.GetOptimizationScenarioNames();
          });
          var experiments = ControllerService.withControllerService<IEnumerable<string>>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.GetExperiments(u);
          });
          foreach (var experiment in experiments)
            model.Experiments.Add(experiment);
          foreach (var scenario in scenarios)
            model.Scenarios.Add(scenario);
          return View(model);
        }

        [HttpPost]
        public JsonResult Save(ExperimentModel model) {
          var experiment = new Experiment() { Name = model.Name };
          var stack = new Stack<Node>();

          foreach (var node in model.Experiment.children)
            stack.Push(node);

          while (stack.Count > 0) {
            var currentNode = stack.Pop();
            // add children to stack
            if (currentNode.children != null)
              foreach (var node in currentNode.children)
                stack.Push(node);

            // TODO: Do we really need a tree of scenarios? Or is it a simple list?
            // push currentNode to experiment
            if (currentNode.title == "Experiment")
              continue;
            experiment.Scenarios.Add(new OptimizationScenario() { Id = currentNode.title });
          }

          var ok = ControllerService.withControllerService<bool>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.SaveExperiment(u, experiment);
          });

          if (ok)
            return Json(new { success = true, data = model.Name });
          return Json(new { success = false, error = "Experiment already exists!" });
        }

        public ActionResult Edit() {
          var model = new ExperimentViewModel();
          var experiments = ControllerService.withControllerService<IEnumerable<string>>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.GetExperiments(u);
          });
          foreach (var experiment in experiments)
            model.Experiments.Add(experiment);
          return View(model);
        }

        public ActionResult DeleteExperiment(string experiment) {
          ControllerService.withControllerService<bool>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.DeleteExperiment(u, experiment);
          });
          return RedirectToAction("Edit");
        }
    }
}
